///////////////////////////////////////////////////////////////////////////////
//                                                         
// Tokenizer.cc
// ------------
// Tokenizer implementation module
//                                               
// Design and Implementation by Bjoern Lemke               
//                                                         
// (C)opyright 2000-2005 Bjoern Lemke                        
//                                                         
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// IMPLEMENTATION MODULE
//
// Class: 
// 
// Description:
//
// Status: IMPLEMENTED
///////////////////////////////////////////////////////////////////////////////


// BASE INCLUDES
#include "Chain.h"
#include "Tokenizer.h"
#include "Exception.h"

Tokenizer::Tokenizer(const Chain& tokenStream, const Chain& sep, char quote, char esc)
{
    _tokenStream = tokenStream;
    _sep = sep;
    _esc = esc;
    _quote = quote;
    _lower = 0;
    _upper = 0;
}

Tokenizer::~Tokenizer()
{
}

bool Tokenizer::nextToken(Chain& token)
{
    if ( _tokenStream.length() <= _upper)
	return false;

    int si;

    bool sepFound = true;
    bool isQuoted=false;

    // skip through sep

    while (sepFound == true)
    {
	sepFound = false;

	si=0;
	while (sepFound == false && si<_sep.length()-1)
	{
	    char c;
	    c = _sep[si];

	    if (_tokenStream[_upper] == c)
	    {
		sepFound = true;
	    }
	    si++;
	}
	if (sepFound == true)
	{
	    _upper++;
	}
    }

    _lower = _upper;

    while (sepFound == false && _tokenStream[_upper])
    {
	if ( _tokenStream[_upper] == _esc )
	{
	    _upper++;
	    if ( _tokenStream[_upper] )
		_upper++;
	    else
		throw Exception(EXLOC, Chain("Unexpected end of token steam"));
	    
	}
	else if ( _tokenStream[_upper] )
	{
	    if ( isQuoted )
	    {
		if (_tokenStream[_upper] == _quote)
		    isQuoted = false;
	    }
	    else if ( _tokenStream[_upper] == _quote)
	    {
		isQuoted = true;
	    }
	    else
	    {
		si=0;
		while ( sepFound == false  && si<_sep.length()-1)
		{			      
		    char c;	
		    c = _sep[si];
		    if (_tokenStream[_upper] == c)
		    {
			sepFound = true;
		    }
		    si++;
		}
	    }
	    if (sepFound == false)
	    {
		_upper++;
	    }
	}
    }

    if (_lower == _upper)
    {
	return false;
    }

    token = _tokenStream.subChain(_lower+1, _upper);

    _lower = _upper;
    
    return true;
}

bool Tokenizer::getTail(Chain& token)
{
  
  if (_tokenStream.length() > _lower+1)
  {
      token = _tokenStream.subChain(_lower+1, _tokenStream.length());
      return true;
  }
  return false;

}

int Tokenizer::numAhead() const
{
    return _tokenStream.length() - _lower - 1;
}


Tokenizer& Tokenizer::operator = (const Tokenizer& t)
{
    _tokenStream = t._tokenStream;
    _sep = t._sep;
    _esc = t._esc;
    _quote = t._quote;
    _lower = t._lower;
    _upper = t._upper;
    return (*this);
}





